@@ -41,12 +41,12 @@
 #include <coreplugin/progressmanager/futureprogress.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 
-#include <cpptools/clangdiagnosticconfigsmodel.h>
-#include <cpptools/compileroptionsbuilder.h>
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cppprojectfile.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/projectinfo.h>
+#include <cppeditor/clangdiagnosticconfigsmodel.h>
+#include <cppeditor/compileroptionsbuilder.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cppprojectfile.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/projectinfo.h>
 
 #include <projectexplorer/abi.h>
 #include <projectexplorer/buildconfiguration.h>
@@ -67,51 +67,12 @@
 #include <QAction>
 #include <QLoggingCategory>
 
-using namespace CppTools;
+using namespace CppEditor;
 using namespace ProjectExplorer;
 using namespace Utils;
 
 static Q_LOGGING_CATEGORY(LOG, "qtc.clangtools.runcontrol", QtWarningMsg)
 
-static QStringList splitArgs(QString &argsString)
-{
-    QStringList result;
-    Utils::QtcProcess::ArgIterator it(&argsString);
-    while (it.next())
-        result.append(it.value());
-    return result;
-}
-
-static QStringList extraOptions(const char *environment)
-{
-    if (!qEnvironmentVariableIsSet(environment))
-        return QStringList();
-    QString arguments = QString::fromLocal8Bit(qgetenv(environment));
-    return splitArgs(arguments);
-}
-
-static QStringList extraClangToolsPrependOptions()
-{
-    constexpr char csaPrependOptions[] = "QTC_CLANG_CSA_CMD_PREPEND";
-    constexpr char toolsPrependOptions[] = "QTC_CLANG_TOOLS_CMD_PREPEND";
-    static const QStringList options = extraOptions(csaPrependOptions)
-            + extraOptions(toolsPrependOptions);
-    if (!options.isEmpty())
-        qWarning() << "ClangTools options are prepended with " << options.toVector();
-    return options;
-}
-
-static QStringList extraClangToolsAppendOptions()
-{
-    constexpr char csaAppendOptions[] = "QTC_CLANG_CSA_CMD_APPEND";
-    constexpr char toolsAppendOptions[] = "QTC_CLANG_TOOLS_CMD_APPEND";
-    static const QStringList options = extraOptions(csaAppendOptions)
-            + extraOptions(toolsAppendOptions);
-    if (!options.isEmpty())
-        qWarning() << "ClangTools options are appended with " << options.toVector();
-    return options;
-}
-
 namespace ClangTools {
 namespace Internal {
 
@@ -154,34 +115,32 @@ private:
      bool m_success = false;
 };
 
-static AnalyzeUnits toAnalyzeUnits(const FileInfos &fileInfos, const FilePath &clangResourceDir,
-                                   const QString &clangVersion)
+AnalyzeUnit::AnalyzeUnit(const FileInfo &fileInfo,
+                         const FilePath &clangIncludeDir,
+                         const QString &clangVersion)
 {
-    AnalyzeUnits unitsToAnalyze;
-    const UsePrecompiledHeaders usePrecompiledHeaders = CppTools::getPchUsage();
-    for (const FileInfo &fileInfo : fileInfos) {
-        CompilerOptionsBuilder optionsBuilder(*fileInfo.projectPart,
-                                              UseSystemHeader::No,
-                                              UseTweakedHeaderPaths::Yes,
-                                              UseLanguageDefines::No,
-                                              UseBuildSystemWarnings::No,
-                                              clangVersion,
-                                              clangResourceDir.toString());
-        QStringList arguments = extraClangToolsPrependOptions();
-        arguments.append(optionsBuilder.build(fileInfo.kind, usePrecompiledHeaders));
-        arguments.append(extraClangToolsAppendOptions());
-        unitsToAnalyze << AnalyzeUnit(fileInfo.file.toString(), arguments);
-    }
-
-    return unitsToAnalyze;
+    CompilerOptionsBuilder optionsBuilder(*fileInfo.projectPart,
+                                          UseSystemHeader::No,
+                                          UseTweakedHeaderPaths::Tools,
+                                          UseLanguageDefines::No,
+                                          UseBuildSystemWarnings::No,
+                                          clangVersion,
+                                          clangIncludeDir);
+    file = fileInfo.file.toString();
+    arguments = extraClangToolsPrependOptions();
+    arguments.append(optionsBuilder.build(fileInfo.kind, CppEditor::getPchUsage()));
+    arguments.append(extraClangToolsAppendOptions());
 }
 
-AnalyzeUnits ClangToolRunWorker::unitsToAnalyze(const FilePath &clangResourceDir,
+AnalyzeUnits ClangToolRunWorker::unitsToAnalyze(const FilePath &clangIncludeDir,
                                                 const QString &clangVersion)
 {
-    QTC_ASSERT(m_projectInfo.isValid(), return AnalyzeUnits());
+    QTC_ASSERT(m_projectInfo, return AnalyzeUnits());
 
-    return toAnalyzeUnits(m_fileInfos, clangResourceDir, clangVersion);
+    AnalyzeUnits units;
+    for (const FileInfo &fileInfo : m_fileInfos)
+        units << AnalyzeUnit(fileInfo, clangIncludeDir, clangVersion);
+    return units;
 }
 
 static QDebug operator<<(QDebug debug, const Utils::Environment &environment)
@@ -201,7 +160,7 @@ static QDebug operator<<(QDebug debug, const AnalyzeUnits &analyzeUnits)
 
 ClangToolRunWorker::ClangToolRunWorker(RunControl *runControl,
                                        const RunSettings &runSettings,
-                                       const CppTools::ClangDiagnosticConfig &diagnosticConfig,
+                                       const CppEditor::ClangDiagnosticConfig &diagnosticConfig,
                                        const FileInfos &fileInfos,
                                        bool buildBeforeAnalysis)
     : RunWorker(runControl)
@@ -220,7 +179,7 @@ ClangToolRunWorker::ClangToolRunWorker(RunControl *runControl,
     }
 
     Target *target = runControl->target();
-    m_projectInfoBeforeBuild = CppTools::CppModelManager::instance()->projectInfo(target->project());
+    m_projectInfoBeforeBuild = CppEditor::CppModelManager::instance()->projectInfo(target->project());
 
     BuildConfiguration *buildConfiguration = target->activeBuildConfiguration();
     QTC_ASSERT(buildConfiguration, return);
@@ -257,11 +216,15 @@ void ClangToolRunWorker::start()
 
     const QString &toolName = tool()->name();
     Project *project = runControl()->project();
-    m_projectInfo = CppTools::CppModelManager::instance()->projectInfo(project);
+    m_projectInfo = CppEditor::CppModelManager::instance()->projectInfo(project);
+    if (!m_projectInfo) {
+        reportFailure(tr("No code model data available for project."));
+        return;
+    }
     m_projectFiles = Utils::toSet(project->files(Project::AllFiles));
 
     // Project changed in the mean time?
-    if (m_projectInfo.configurationOrFilesChanged(m_projectInfoBeforeBuild)) {
+    if (m_projectInfo->configurationOrFilesChanged(*m_projectInfoBeforeBuild)) {
         // If it's more than a release/debug build configuration change, e.g.
         // a version control checkout, files might be not valid C++ anymore
         // or even gone, so better stop here.
@@ -280,7 +243,7 @@ void ClangToolRunWorker::start()
         return;
     }
 
-    const Utils::FilePath projectFile = m_projectInfo.project()->projectFilePath();
+    const Utils::FilePath projectFile = m_projectInfo->projectFilePath();
     appendMessage(tr("Running %1 on %2 with configuration \"%3\".")
                       .arg(toolName)
                       .arg(projectFile.toUserOutput())
@@ -288,10 +251,12 @@ void ClangToolRunWorker::start()
                   Utils::NormalMessageFormat);
 
     // Collect files
-    const auto clangResourceDirAndVersion =
-            getClangResourceDirAndVersion(runControl()->runnable().executable);
-    const AnalyzeUnits unitsToProcess = unitsToAnalyze(clangResourceDirAndVersion.first,
-                                                       clangResourceDirAndVersion.second);
+    const auto clangIncludeDirAndVersion =
+            getClangIncludeDirAndVersion(runControl()->runnable().command.executable());
+    const AnalyzeUnits unitsToProcess = unitsToAnalyze(clangIncludeDirAndVersion.first,
+                                                       clangIncludeDirAndVersion.second);
+    qCDebug(LOG) << Q_FUNC_INFO << runControl()->runnable().command.executable()
+                 << clangIncludeDirAndVersion.first << clangIncludeDirAndVersion.second;
     qCDebug(LOG) << "Files to process:" << unitsToProcess;
 
     m_queue.clear();
@@ -335,7 +300,7 @@ void ClangToolRunWorker::start()
 
 void ClangToolRunWorker::stop()
 {
-    for (ClangToolRunner *runner : m_runners) {
+    for (ClangToolRunner *runner : qAsConst(m_runners)) {
         QObject::disconnect(runner, nullptr, this, nullptr);
         delete runner;
     }
@@ -403,8 +368,12 @@ void ClangToolRunWorker::onRunnerFinishedWithSuccess(const QString &filePath)
     } else {
         if (!m_filesNotAnalyzed.contains(filePath))
             m_filesAnalyzed.insert(filePath);
-        if (!diagnostics.isEmpty())
-            tool()->onNewDiagnosticsAvailable(diagnostics);
+        if (!diagnostics.isEmpty()) {
+            // do not generate marks when we always analyze open files since marks from that
+            // analysis should be more up to date
+            const bool generateMarks = !m_runSettings.analyzeOpenFiles();
+            tool()->onNewDiagnosticsAvailable(diagnostics, generateMarks);
+        }
     }
 
     handleFinished();
@@ -420,7 +389,6 @@ void ClangToolRunWorker::onRunnerFinishedWithFailure(const QString &errorMessage
 
     auto *toolRunner = qobject_cast<ClangToolRunner *>(sender());
     const QString fileToAnalyze = toolRunner->fileToAnalyze();
-    const QString outputFilePath = toolRunner->outputFilePath();
 
     m_filesAnalyzed.remove(fileToAnalyze);
     m_filesNotAnalyzed.insert(fileToAnalyze);
@@ -454,7 +422,7 @@ void ClangToolRunWorker::finalize()
 {
     const QString toolName = tool()->name();
     if (m_filesNotAnalyzed.size() != 0) {
-        appendMessage(tr("Error: Failed to analyze %1 files.").arg(m_filesNotAnalyzed.size()),
+        appendMessage(tr("Error: Failed to analyze %n files.", nullptr, m_filesNotAnalyzed.size()),
                       ErrorMessageFormat);
         Target *target = runControl()->target();
         if (target && target->activeBuildConfiguration() && !target->activeBuildConfiguration()->buildDirectory().exists()
